"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.showArmWarning = void 0;
const path = require("path");
const parseArgs = require("yargs-parser");
const chalk = require("chalk");
const tempy = require("tempy");
const patch_apk_1 = require("./patch-apk");
const patch_app_bundle_1 = require("./patch-app-bundle");
const apktool_1 = require("./tools/apktool");
const uber_apk_signer_1 = require("./tools/uber-apk-signer");
const { version } = require('../package.json');
async function main() {
    const args = parseArgs(process.argv.slice(2), {
        string: ['apktool'],
        boolean: ['help', 'wait', 'skip-patches', 'debuggable'],
    });
    if (args.help) {
        showHelp();
        process.exit();
    }
    const [input] = args._;
    if (!input) {
        showHelp();
        process.exit(1);
    }
    const inputPath = path.resolve(process.cwd(), input);
    const fileExtension = path.extname(input);
    const baseName = path.basename(input, fileExtension);
    const outputName = `${baseName}-patched${fileExtension}`;
    const outputPath = path.resolve(path.dirname(inputPath), outputName);
    let taskFunction;
    switch (fileExtension) {
        case '.apk':
            taskFunction = patch_apk_1.default;
            break;
        case '.xapk':
            taskFunction = patch_app_bundle_1.patchXapkBundle;
            break;
        case '.apks':
        case '.zip':
            taskFunction = patch_app_bundle_1.patchApksBundle;
            break;
        default:
            showSupportedExtensions();
    }
    const tmpDir = tempy.directory({ prefix: 'apk-mitm-' });
    process.chdir(tmpDir);
    const apktool = new apktool_1.default({
        frameworkPath: path.join(tmpDir, 'framework'),
        customPath: args.apktool,
    });
    const uberApkSigner = new uber_apk_signer_1.default();
    showVersions({ apktool, uberApkSigner });
    console.log(chalk.dim(`  Using temporary directory:\n  ${tmpDir}\n`));
    taskFunction({
        inputPath,
        outputPath,
        tmpDir,
        apktool,
        uberApkSigner,
        wait: args.wait,
        skipPatches: args.skipPatches,
        debuggable: args.debuggable,
    })
        .run()
        .then(context => {
        if (taskFunction === patch_apk_1.default && context.usesAppBundle) {
            patch_apk_1.showAppBundleWarning();
        }
        console.log(chalk `\n  {green.inverse  Done! } Patched file: {bold ./${outputName}}\n`);
    })
        .catch((error) => {
        const message = getErrorMessage(error, { tmpDir });
        console.error([
            '',
            chalk `  {red.inverse.bold  Failed! } An error occurred:`,
            '',
            message,
            '',
            `  The full logs of all commands are available here:`,
            `  ${path.join(tmpDir, 'logs')}`,
            '',
        ].join('\n'));
        if (process.arch.startsWith('arm'))
            showArmWarning();
        process.exit(1);
    });
}
function getErrorMessage(error, { tmpDir }) {
    if (error.all)
        return formatCommandError(error.all, { tmpDir });
    return error.stack;
}
function formatCommandError(error, { tmpDir }) {
    return (error
        // Replace mentions of the (sometimes very long) temporary directory path
        .replace(new RegExp(tmpDir, 'g'), chalk `{bold <tmp_dir>}`)
        // Highlight (usually relevant) warning lines in Apktool output
        .replace(/^W: .+$/gm, line => chalk `{yellow ${line}}`)
        // De-emphasize Apktool info lines
        .replace(/^I: .+$/gm, line => chalk `{dim ${line}}`)
        // De-emphasize (not very helpful) Apktool "could not exec" error message
        .replace(/^.+brut\.common\.BrutException: could not exec.+$/gm, line => chalk `{dim ${line}}`));
}
function showHelp() {
    console.log(chalk `
  $ {bold apk-mitm} <path-to-apk/xapk/apks>
      {dim {bold --wait} Wait for manual changes before re-encoding {gray.italic (optional)}}
      {dim {bold --debuggable} Make the patched app debuggable {gray.italic (optional)}}
      {dim {bold --apktool} Path to custom Apktool.jar {gray.italic (optional)}}
      {dim {bold --skip-patches} Don't apply any patches {gray.italic (optional)}}
  `);
}
function showSupportedExtensions() {
    console.log(chalk `{yellow
  It looks like you tried running {bold apk-mitm} with an unsupported file type!

  Only the following file extensions are supported: {bold .apk}, {bold .xapk}, and {bold .apks} (or {bold .zip})
  }`);
    process.exit(1);
}
function showVersions({ apktool, uberApkSigner, }) {
    console.log(chalk `
  {dim ╭} {blue {bold apk-mitm} v${version}}
  {dim ├ {bold apktool} ${apktool.version.name}
  ╰ {bold uber-apk-signer} ${uberApkSigner.version.name}}
  `);
}
function showArmWarning() {
    console.log(chalk `{yellow
  {inverse.bold  NOTE }

  {bold apk-mitm} doesn't officially support ARM-based devices (like Raspberry Pi's)
  at the moment, so the error above might be a result of that. Please try
  patching this APK on a device with a more common CPU architecture like x64
  before reporting an issue.
  }`);
}
exports.showArmWarning = showArmWarning;
main();
